#!/usr/bin/env python

# saturation_h_m_l_trial.py
#
# License: GPLv3
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY# without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# To view a copy of the GNU General Public License
# visit: http://www.gnu.org/licenses/gpl.html
#
#
# ------------
#| Change Log |
# ------------
# Version 1 02/11/2021
# 
#
#
#import Image
from gimpfu import *
def python_saturation_h_m_l_trial(image, layer, transition):
	pdb.gimp_image_undo_group_start(image)
	pdb.gimp_context_push()
	
	frozen = pdb.gimp_image_undo_freeze(image)																#freeze undo speeds up plug-in &
																											#reduces memory usage
	
	if not pdb.gimp_selection_is_empty(image):																#check for existing selection
		old_select = pdb.gimp_selection_save(image)															#if exists - save to channel
	pdb.gimp_selection_none(image)																			#remove any existing selection
	
	layerNew = pdb.gimp_layer_new_from_drawable(layer, image)												#layerNew is existing name in loop
	pdb.gimp_image_insert_layer(image, layerNew, None, 0)
	
	width   = image.width
	height  = image.height
	type    = RGB_IMAGE
	opacity = 100
	mode    = NORMAL_MODE
	
	layer_copy = pdb.gimp_layer_new(image, width, height, type, "layer_copy", opacity, mode)				#new empty layer
	image.add_layer(layer_copy, 0)																			#layer_copy is existing name in loop
	
	pdb.gimp_image_remove_layer(image, layer)																#no longer needed - just so I could
																											#re-use loop routine with orig. names
																											
	decomposed_image, not_used1, not_used2, not_used3 = pdb.plug_in_decompose(image, layerNew, "HSV", 1)	#decompose image HSV mode
	
	display = pdb.gimp_display_new(decomposed_image)
	
	saturation_layer = decomposed_image.layers[1]															#get saturation layer from HSV

	pdb.gimp_selection_all(decomposed_image)
	pdb.gimp_edit_copy(saturation_layer)																	#copy to buffer to use later
	
	pdb.gimp_display_delete(display)	
	
	floating_sel = pdb.gimp_edit_paste(layer_copy, TRUE)													#copy saturation to layer_copy
		
	mask = pdb.gimp_layer_create_mask(layerNew, ADD_MASK_WHITE)												#add empty mask before loop
	pdb.gimp_layer_add_mask(layerNew, mask)
	pdb.gimp_floating_sel_anchor(floating_sel)
	
	pdb.gimp_layer_set_edit_mask(layerNew, FALSE) 															#make layer active
	
	splinePoints0 = [[0.000,1.0, 0.167,1.0, 0.333,1.0, 0.500,0.0, 0.666,0.0, 0.833,0.0, 1.000,0.0],			#hard curves to apply to masks
				[0.000,0.0, 0.167,0.0, 0.333,1.0, 0.500,1.0, 0.666,1.0, 0.833,0.0, 1.000,0.0],
				[0.000,0.0, 0.167,0.0, 0.333,0.0, 0.500,0.0, 0.666,1.0, 0.833,1.0, 1.000,1.0]]
				
	splinePoints1 = [[0.000,1.0, 0.167,1.0, 0.333,1.0, 0.500,0.3, 0.666,0.0, 0.833,0.0, 1.000,0.0],			#medium curves to apply to masks
				[0.000,0.0, 0.167,0.3, 0.333,1.0, 0.500,1.0, 0.666,1.0, 0.833,0.3, 1.000,0.0],
				[0.000,0.0, 0.167,0.0, 0.333,0.0, 0.500,0.3, 0.666,1.0, 0.833,1.0, 1.000,1.0]]
				
	splinePoints2 = [[0.000,1.0, 0.167,1.0, 0.333,1.0, 0.500,0.6, 0.666,0.0, 0.833,0.0, 1.000,0.0],			#soft curves to apply to masks
				[0.000,0.0, 0.167,0.6, 0.333,1.0, 0.500,1.0, 0.666,1.0, 0.833,0.6, 1.000,0.0],
				[0.000,0.0, 0.167,0.0, 0.333,0.0, 0.500,0.6, 0.666,1.0, 0.833,1.0, 1.000,1.0]]
				
	#############START OF REPEATED SECTION#####################
	
	layerNum = (1)																							#integer index for layerName
	
	if transition == 0:
		splinePoints = splinePoints0
		
	elif transition == 1:
		splinePoints = splinePoints1
		
	elif transition == 2:	
		splinePoints = splinePoints2
				
	for spline in splinePoints:																				#first row of splinePoints used when
																											#layerNum=(0), second when (1), etc.		
	
		layer_copy1 = pdb.gimp_layer_new_from_drawable(layer_copy, image)									#make a layer for applying curve
		pdb.gimp_image_insert_layer(image, layer_copy1, None, 0)
		
		pdb.gimp_drawable_curves_spline(layer_copy1, HISTOGRAM_VALUE, len(spline), spline)					#cycle through splinePoints list
		
		pdb.gimp_selection_all(image)																		#copy to buffer		
		pdb.gimp_edit_copy(layer_copy1)	
		
		pdb.gimp_image_remove_layer(image, layer_copy1)														#remove splined layer									
 		floating_sel=pdb.gimp_edit_paste(mask, TRUE)														#paste into mask										
		pdb.gimp_floating_sel_anchor(floating_sel)

 		pdb.gimp_item_set_visible(layerNew, TRUE)															#make layer active											
 		pdb.gimp_selection_none(image)		
		
		layer_add = pdb.gimp_layer_new_from_drawable(layerNew, image)										#copy white masked layer for
		pdb.gimp_image_insert_layer(image, layer_add, None, 0)												#next time around loop 		
 		
 		if layerNum == 1:
 			layerName = "Low"
 			
 		if layerNum == 2:
 			layerName = "Medium"
 			
 		if layerNum == 3:
 			layerName = "High" 			
 		
 		pdb.gimp_item_set_name(layer_add, layerName) 		
 		
 		layerNum+=1		
		
	###############################END OF REPEATED SECTION##################################	
	
	pdb.gimp_image_remove_layer(image, layer_copy)
	pdb.gimp_image_remove_layer(image, layerNew)
	
	thawed = pdb.gimp_image_undo_thaw(image)																#re-enable undo stack	
	
	pdb.gimp_context_pop()
	pdb.gimp_image_undo_group_end(image)
	pdb.gimp_displays_flush()

register(
	"python_saturation_h_m_l_trial",
	"Saturation H M L Trial",
	"Saturation H M L Trial",
	"David Marsden",
	"David Marsden",
	"October 2021",
	"<Image>/Python-Fu/Saturation H M L Trial...",
	"RGB*", 
	[
		(PF_OPTION, "transition", "Transition - Overlap", 0, ("Hard", "Medium", "Soft"))
	],
	[],
	python_saturation_h_m_l_trial)

main()
